class CORSMiddleware:
    async def __call__(self, scope, receive, send):
        if scope["method"] == "OPTIONS":
            await send(
                {
                    "type": "http.response.start",
                    "status": 200,
                    "headers": [
                        [b"access-control-allow-origin", b"null"],
                        [
                            b"access-control-allow-headers",
                            b"access-control-allow-headers,access-control-allow-methods,access-control-allow-origin"
                        ],
                    ],
                }
            )
            await send({"type": "http.response.body", "body": b"", "more_body": False})



import json


class JetRouter:
    def __init__(self, send):
        self.send = send

    async def send_message(self, message: str):
        await self.send(
            {
                "type": "http.response.start",
                "status": 200,
                "headers": [
                    [b"content-type", b"application/json"],
                    [b"access-control-allow-origin", b"*"],
                ],
            }
        )
        await self.send(
            {
                "type": "http.response.body",
                "body": json.dumps(message).encode(),
            }
        )

    async def not_found(self):
        await self.send(
            {
                "type": "http.response.start",
                "status": 404,
                "headers": [
                    [b"content-type", b"application/json"],
                    [b"access-control-allow-origin", b"*"],
                ],
            }
        )
        await self.send(
            {
                "type": "http.response.body",
                "body": b"Not Found",
            }
        )

    async def unsupported_method(self):
        await self.send(
            {
                "type": "http.response.start",
                "status": 405,
                "headers": [
                    [b"content-type", b"application/json"],
                    [b"access-control-allow-origin", b"*"],
                ],
            }
        )
        await self.send(
            {
                "type": "http.response.body",
                "body": b"Method Not Allowed",
            }
        )

    async def middleware_error(self):
        await self.send(
            {
                "type": "http.response.start",
                "status": 500,
                "headers": [
                    [b"content-type", b"application/json"],
                    [b"access-control-allow-origin", b"*"],
                ],
            }
        )
        await self.send(
            {
                "type": "http.response.body",
                "body": b"Middleware Error"
            }
        )



import json


class PrivateJet:
    routers = []
    middlewares = []

    async def read_body(self, receive):
        """
        Read and return the entire body from an incoming ASGI message.
        """
        body = b''
        more_body = True

        while more_body:
            message = await receive()
            body += message.get('body', b'')
            more_body = message.get('more_body', False)

        return json.loads(body)

    async def start(self, receive, scope, send):
        self.scope = scope
        assert scope["type"] == "http"
        route = scope["path"]
        for router in self.routers:
            if route == router["prefix"] or route == router["prefix"] + "/" or route.startswith(router["prefix"] + "/"):
                request = {
                    "method": scope["method"],
                    "path": scope["path"],
                    "query_string": scope["query_string"],
                    "headers": scope["headers"],
                }
                if request["method"] in ["POST", "PUT", "PATCH"]:
                    request["body"] = await self.read_body(receive)
                try:
                    for middleware in self.middlewares:
                        try:
                            await middleware()(scope, receive, send)
                        except Exception:
                            await router["router"](send).middleware_error()
                    match scope["method"]:
                        case "GET":
                            if route == router["prefix"] or route == router["prefix"] + "/":
                                await router["router"](send).get(request)
                            else:
                                await router["router"](send).get_one(request)
                        case "POST":
                            await router["router"](send).post(request)
                        case "PUT":
                            await router["router"](send).put(request)
                        case "PATCH":
                            await router["router"](send).patch(request)
                        case "DELETE":
                            await router["router"](send).delete(request)
                except AttributeError:
                    await router["router"](send).unsupported_method()
            else:
                await router["router"](send).not_found()

    async def add_router(self, router):
        if router["prefix"] not in [
            router_obj["prefix"] for router_obj in self.routers
        ]:
            self.routers.append(
                {
                    "prefix": router["prefix"],
                    "router": router["router"]
                }
            )

    async def add_middleware(self, middleware_class):
        if middleware_class not in self.middlewares:
            self.middlewares.append(middleware_class)



from setuptools import setup, find_packages

classifiers = [
  'Development Status :: 5 - Production/Stable',
  'Intended Audience :: Education',
  'Operating System :: Microsoft :: Windows :: Windows 11',
  'License :: OSI Approved :: MIT License',
  'Programming Language :: Python :: 3.11'
]

setup(
  name='privatejet',
  version='0.0.2',
  description='',
  long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.txt').read(),
  long_description_content_type="text/markdown",
  url='',  
  author='m-moein-98',
  author_email='moein1475963.mmz@gmail.com',
  license='OSI Approved :: MIT License', 
  classifiers=classifiers,
  keywords='python backend framework private-jet-framework', 
  packages=find_packages(),
  install_requires=[''] 
)